/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_COLLECTION_MANAGER_H
#define NEST_COLLECTION_MANAGER_H

#include "nest_common.h"
#include <string>
#include "time.h"

class 
NestCollection {

public:
    NestCollection( const string &name, const string &path, ClassAd *ad );
    NestCollection( ClassAd *ad );
    NestCollection( void );
    ~NestCollection();

	NestReplyStatus Init( const string &name, const string &path, ClassAd *ad = NULL );
	void Clear( void );
	bool NestCollection::buildCollectionFromAd( ClassAd *ad );

	bool isValid( void ) const { return valid; };
    const string * getName( ) const { return &name; };
    const string * getPath( ) const { return &path; };

	time_t getMtime( ) const { return mtime; };
	const string * getVersion( ) const { return &version; };
	const string * getComment( ) const { return &comment; };

	NestReplyStatus setMtime( time_t mtime );
	NestReplyStatus setVersion( const string &version );
	NestReplyStatus setVersion( const char *version );
	NestReplyStatus setComment( const string &comment );
	NestReplyStatus setComment( const char *comment );

	// Get / set my ClassAd (for CollectionManager only)
	ClassAd * getAd( void ) { return this->ad; };

private:
	bool	valid;
	string 	name;
    string	path;
	time_t 	mtime;
	string	version;
	string	comment;
	ClassAd	*ad;
};

class
NestCollectionVisitor
{
  public:
	NestCollectionVisitor( void ) { };
	virtual ~NestCollectionVisitor( void ) { };
	virtual NestReplyStatus visit( const NestCollection & ) = NULL;
};

class
NestCollectionManager 
{

public:

    int init( );
	NestReplyStatus initialize(ClassAdCollection *resources);
	NestReplyStatus recover(ClassAdCollection *resources);

    NestReplyStatus createAssociation( const string &name,
									   const string &path );

    NestReplyStatus removeAssociation( const string &name );

    const string * getPath( const string &name );

    NestCollection* getCollection( const string &name );
    bool checkName( const string &name );

	NestReplyStatus updateMtime( NestCollection *col, time_t mtime );
	NestReplyStatus updateVersion( NestCollection *col, const string &version );
	NestReplyStatus updateComment( NestCollection *col, const string &comment );
	NestReplyStatus update( NestCollection *col );

	NestReplyStatus VisitAllCollections( NestCollectionVisitor &visitor );

  private:
	ClassAdCollection		*collection_list;
	ClassAdParser			parser;

	LocalCollectionQuery    query;
	ViewName                collection_view;

	// Private methods
	NestReplyStatus setupView( ClassAdCollection *resources );
	
};

#endif
