/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_FILECACHE_MODEL_H
#define NEST_FILECACHE_MODEL_H

#include <stdio.h>
#include <assert.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "nest_common.h"
#include "nest_filecache_entry.h"
#include "nest_hash_table.h"
#include "nest_protocol.h"

#define INFINITE_IMPORTANCE -1

typedef NestReplyStatus (*removeCallback) ( const char *file );

class nestFilecacheModel {

public:

    nestFilecacheModel( );
    
    nestFilecacheModel( float megs, int type, 
            removeCallback cb = NULL, bool track_hits = false );

    virtual NestReplyStatus insert( bool *hit, const char *file, 
                                    long size, int weight = 1 );

    virtual bool remove( const char *file );
    
    void refresh( const char *file );

    virtual bool check( const char *file );

    nestFilecacheEntry * check_and_refresh( const char *file );

    void debug( );

    virtual bool grow( float megs );

    virtual bool shrink( float megs );

    virtual bool renameFile( const char *oldname, const char *newname );

    virtual float hit_rate( );
    
    virtual long gethits( );

    virtual long getmisses( );
    
    virtual long getevictions( );
    
    virtual float getsize( );

    virtual long getavail( );

    virtual long getFilesize( const char *file );

    virtual void setFilesize( const char *file, long size );

    virtual void writeAd( nestProtocol *ad );

    virtual NestReplyStatus format();

    virtual bool rememberLength( const char *file, long *length );
    
private:

    nestFilecacheEntry *head, *tail;
    removeCallback removeCb;
    struct nest_hash_table *ht;

    long size, avail;   // in KB
    long entries;
    bool use_hashtable;
    bool log_misses;
    int  type;

    struct nest_hash_table *infinite_cache;
    nestProtocol *cacheLog;
    unsigned long hits, compulsory_misses, evictions, capacity_misses;

    void logMiss( const char *file );
    long make_space( long size );
    long freeEntry( nestFilecacheEntry * entry );
    bool validityCheck();

    nestFilecacheEntry * pushEntry( nestFilecacheEntry * entry );
    nestFilecacheEntry * removeEntry( nestFilecacheEntry * entry );
    nestFilecacheEntry * findEntry( const char *file );
};

#endif
