/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#include "nest_filecache_model_infinite.h"


// constructor
nestFilecacheModelInfinite::nestFilecacheModelInfinite( int hashsize ) {
    this->infinite_cache = nest_hash_table_create( hashsize, NULL );
    assert( this->infinite_cache );
    this->hits = this->misses = this->size = 0; 
}

void
nestFilecacheModelInfinite::writeAd( nestProtocol *ad ) {
}
    
bool 
nestFilecacheModelInfinite::rememberLength( const char *file, long *length ) {
    nest_warn( __FILE__, __FUNCTION__, "Called a method that does nothing: %s:%s\n",
            __FILE__, __FUNCTION__ );
    return true;
}

// returns whether file was already in cache
bool
nestFilecacheModelInfinite::remove( const char *file ) {
    bool ret = check( file );
    nest_hash_table_remove( infinite_cache, file );
    return ret;
}

float
nestFilecacheModelInfinite::hit_rate( ) {
    return (float)hits/(hits + misses);
}

long
nestFilecacheModelInfinite::getevictions() {
    return 0;
}

long
nestFilecacheModelInfinite::gethits() {
    return hits;
}

long
nestFilecacheModelInfinite::getmisses() {
    return misses;
}

// returns whether file was already in cache
// this is called when tmp file is complete, so don't actually
// free the temp file name from the infinite cache...
bool
nestFilecacheModelInfinite::renameFile( const char *old, const char *newname ) {
    bool ret = check( newname );
    nest_hash_table_insert( infinite_cache, newname, infinite_cache );
    return ret;
}

NestReplyStatus
nestFilecacheModelInfinite::insert( bool *hit, const char *file, long filesize )
{
    if ( check( file ) ) {
        if ( hit != NULL ) *hit = true;
        hits++;
    } else {
        if ( hit != NULL ) *hit = false;
        misses++;
        nest_hash_table_insert( infinite_cache, file, infinite_cache );
        size += filesize / 1024;
    }
    return NEST_SUCCESS; 
}
    
bool 
nestFilecacheModelInfinite::shrink( float megs ) {
    nest_warn( __FILE__, __FUNCTION__, "Called a method that does nothing: %s:%s\n",
            __FILE__, __FUNCTION__ );
    return true;
}

bool
nestFilecacheModelInfinite::grow( float megs ) {
    nest_warn( __FILE__, __FUNCTION__, "Called a method that does nothing: %s:%s\n",
            __FILE__, __FUNCTION__ );
    return true;
}

float 
nestFilecacheModelInfinite::getsize( ) {
    return ((float)size / 1024); 
}

long
nestFilecacheModelInfinite::getavail( ) {
    return 0;
}

long
nestFilecacheModelInfinite::getentries() {
    long entries = misses; 
    assert( entries == nest_hash_table_size( infinite_cache ) );
    return entries;
}

bool
nestFilecacheModelInfinite::check( const char *file ) {
    return ( nest_hash_table_lookup( infinite_cache, file ) != NULL );
}

NestReplyStatus
nestFilecacheModelInfinite::format( ) {
    return NEST_SUCCESS;
}

long
nestFilecacheModelInfinite::getFilesize( const char *file ) {
    nest_warn( __FILE__, __FUNCTION__, "Called a method that does nothing: %s:%s\n",
            __FILE__, __FUNCTION__ );
    return 0;
}

void
nestFilecacheModelInfinite::setFilesize( const char *file, long size ) {
    nest_warn( __FILE__, __FUNCTION__, "Called a method that does nothing: %s:%s\n",
            __FILE__, __FUNCTION__ );
}

void
nestFilecacheModelInfinite::debug( ) {
    nest_warn( __FILE__, __FUNCTION__, "Called a method that does nothing: %s:%s\n",
            __FILE__, __FUNCTION__ );
}
