/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_FILECACHE_MODEL_MGR_H
#define NEST_FILECACHE_MODEL_MGR_H

#include <stdio.h>
#include <assert.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "nest_common.h"
#include "nest_storage_manager.h"
#include "nest_filetransfer.h"
#include "nest_filecache_model.h"

#define DEFAULT_CACHE_SIZE       500
#define DEFAULT_CACHE_THRESHOLD  10 
#define DEFAULT_GROWTH_SIZE      5

enum {
    NEST_CACHE_CLOCK,
    NEST_CACHE_LRU
};

class nestFilecacheModelManager {

public:

    nestFilecacheModelManager( int type );

    int taskDone(
            const char *file, 
            nestFiletransfer *transfer, 
            int concurrency_model,
            int current_tasks );

    bool contains( const char *file );

    bool insert( const char *file, nestFiletransfer *transfer, long filesize );
    
    ssize_t writeAd( nestProtocol *ad );

private:

    nestFilecacheModel *model;

    int           cachetype;
    int           growth_size;
    unsigned long cachehit_guesses;
    unsigned long cachemiss_guesses;
    unsigned long false_hits;
    unsigned long correct_misses;
    unsigned long correct_hits;
    unsigned long false_misses;
    float cache_threshold;
    float cache_initial_size;
    nestProtocol *cacheguesslog;
    nestProtocol *cachesizelog;

};

#endif
