/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent,
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * For contact information, please visit:
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef    _NEST_IDENTITY_MANAGER_H_

#define    _NEST_IDENTITY_MANAGER_H_

#include "nest_common.h"


class NestIdentityManager {

    private:

            ClassAdCollection       *Identities;

            ViewName                identities_view;
            ClassAdParser           parser;
            ClassAdUnParser         unparser;
            LocalCollectionQuery    query;

            string                  prev_userlist;
            string                  userlist;
            string                  prev_denylist;
            string                  denylist;

            NestReplyStatus   get_actual_nest_group(char *actual_nest_group);

            bool              is_user_in_group(const char *user,
                                               const char *group);

            bool              is_group_in_user(const char *user,
                                               const char *group);

            NestReplyStatus   add_user_to_group_members(const char *user,
                                                        const char *group);

            NestReplyStatus   add_group_to_user_groups(const char *user,
                                                        const char *group);

            NestReplyStatus   delete_user_from_group_members(const char *user,
                                                             const char *group);

            NestReplyStatus   delete_group_from_user_groups(const char *user,
                                                            const char *group);

            bool              is_present_in_list(const char *userpool,
                                                 const char *user);

    public:
            NestIdentityManager(const char *userlist,
                                const char *denylist);
            ~NestIdentityManager();

            NestReplyStatus     get_total_nest_users(int *total_nest_users);
            NestReplyStatus     get_total_nest_groups(int *total_nest_groups);
            bool                is_user_present(const char *user);
            bool                is_group_present(const char *group);
            bool                is_active_group(const char *group);
            bool                is_valid_nest_user(const char *user,
                                                   bool is_init = false);

            bool                is_user_access_allowed(const char *user);

            bool                is_group_access_allowed(const char *group,
                                                        const char *user,
                                                        bool allow_members = false);

            NestReplyStatus     get_best_group(string &str_group,
                                               const char *user);

            NestReplyStatus     get_group_owner(const char *group,
                                                char *owner);

            NestReplyStatus     set_default_group(const char *user,
                                                  const char *group,
                                                  bool test_presence = true);

            NestReplyStatus     reset_default_group(const char *user,
                                                    const char *group);

            NestReplyStatus     add_user_in_group(const char *user,
                                                  const char *group,
                                                  const char *cur_user);
            NestReplyStatus     delete_user_in_group(const char *user,
                                                     const char *group,
                                                     const char *cur_user);

            NestReplyStatus     add_user(const char *user,
                                         const char *group,
                                         const char *cur_user,
                                         const char *identity);

            NestReplyStatus     update_user_usage(const char *user,
                                                  double more_size);
            
            NestReplyStatus     update_group_usage(const char *group,
                                                  double more_size);

            NestReplyStatus     delete_user(const char *user,
                                            const char *cur_user);

            NestReplyStatus     add_group(const char *group,
                                          const char *cur_user);

            NestReplyStatus     delete_group(const char *group,
                                             const char *cur_user);

            NestReplyStatus     get_uid(user_id_t *uid, const char *user);
            NestReplyStatus     get_gid(user_id_t *gid, const char *group);
            NestReplyStatus     get_space_used(double *size, 
                                               const char *user, 
                                               bool is_user = true);

            NestReplyStatus     initialize(ClassAdCollection *resources);
            NestReplyStatus     recover(ClassAdCollection *resources);

            NestReplyStatus     shutdown();
            NestReplyStatus     synchronize(bool is_init = false);

};

#endif
