/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_NONBLOCKING_TRANSFERS
#define _NEST_NONBLOCKING_TRANSFERS

#include "nest_concurrency_model.h"
#include "nest_common.h"
#include "nest_client.h"

#define TRANSFER_ITERATOR list< nestNonblockingTransfer *>::iterator

enum{ WRITE_DONE, WRITE_OK, WRITE_INCOMPLETE, READ_DONE, READ_OK, CLIENT_LOST };

class 
nestNonblockingTransfer {

public:

    nestNonblockingTransfer( nestClient *transfer );

    ~nestNonblockingTransfer( );

    static nestNonblockingTransfer* newTransfer( nestClient *client, 
            NestReplyStatus &status );
    
    nestClient* getClient( );
    
    int getSrc( );

    int getDst( );

    int getCsd( );
    int getRealClientSocket();

    int getMaxfd( );

    int getType( );

    int getStatus( );

    int writeDst( );

    int readSrc( );

    int sendInitialAck( );
    bool ackSent( );

    bool sendFinalAck(); 

    bool sendErrorMessage(  const char* filename, const char* func_name, NestReplyStatus error );
    
    void endTransfer( bool status );

    long getLength( ) { return filetransfer->getLength(); }

private:

    // TODO: I think csd is not necessary
    nestProtocol *csd;
    nestProtocol *src;
    nestProtocol *dst;
    int     status;
    int     type;
    long    transferoffset;
    char   *buffer;
    ssize_t bufferoffset;
    ssize_t buffersize;    // how many bytes currently in buffer
    bool     acksent;
    nestClient *client;
    nestFiletransfer *filetransfer;
    struct IO_Measurement *measurements;
};

#endif
