/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#include "nest_common.h"
#include "nest_protocol.h"

nestProtocol::~nestProtocol() {}

bool
nestProtocol::sendInitialAck( ) {
    return sendAck();
}

bool
nestProtocol::sendFinalAck( int transferType ) {
    return sendAck();
}

bool
nestProtocol::persistent() {
    return true;
}

bool
nestProtocol::sendDirectoryList( NestFileInfo *list ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendNestLot( NestLot *rstat ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendLotHandle( NestLotHandle *handle ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendAd( nestAd *ad ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendPwd( const char *pwd ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendVersion( float version ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendProxyList( list <char *> *proxys ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendDiskspace( double avail, double total  ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendBandwidthStatistics( ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendPhysicalName( const char *physical_name ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

NestReplyStatus
nestProtocol::sendFileinfo( NestFileInfo *info ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

NestReplyStatus
nestProtocol::setFileattr( NestFileInfo *info, const char *fname, void *fattr ) {
    PROTOCOL_WARN;
    assert( 0 );
    return NEST_UNKNOWN_ERROR;;
}

bool
nestProtocol::sendAcl( const char *user,
                       int is_user,
                       nest_access_rights_t access ) {
    (void) user;
    (void) is_user;
    (void) access;

    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendAcl( const NestACLList *Acl ) {
    (void) Acl;

    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::sendTransferInfo( class nestRequest *req ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

NestReplyStatus
nestProtocol::sendFilehandle( NestFileInfo *fi ) {
    PROTOCOL_WARN;
    assert( 0 );
    return NEST_NOT_YET_IMPLEMENTED;
}
    
NestReplyStatus 
nestProtocol::sendProxyIndex( DynamicLard *index ) {
    PROTOCOL_WARN;
    assert( 0 );
    return NEST_NOT_YET_IMPLEMENTED;
}
    

NestReplyStatus
nestProtocol::gssAccept( char **username, char **globus_username ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

nestProtocol *
nestProtocol::clone() {
    PROTOCOL_WARN;
    assert( 0 );
    return NULL;
}

NestReplyStatus
nestProtocol::receive_state( int pipe ) {
    // default is a no-op
    // if any particular protocol has state, here is where it can be sent
    // to a process worker
    // see nestProtocol::receive_state
    return NEST_SUCCESS;
}

NestReplyStatus 
nestProtocol::send_state( int pipe ) {
    // see nestProtocol::receive_state
    return NEST_SUCCESS;
}


bool
nestProtocol::sendTimestamp( ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

bool
nestProtocol::endThirdPartyTransfer( int type ) {
    PROTOCOL_WARN;
    assert( 0 );
    return false;
}

// returns -1 on error
ssize_t
nestProtocol::Sendn( const char *buffer, ssize_t n ) {

    ssize_t nleft = n;
    const char *ptr = buffer;
    ssize_t nwritten;

    while ( nleft > 0 ) {
        if ( ( nwritten = Send( ptr, nleft ) ) < 0 ) {
            return -1;
        }
        nleft -= nwritten;
        ptr += nwritten;
    }

    return n;
}

// returns -1 on error
ssize_t
nestProtocol::Send( const string &buffer ) {

    ssize_t nleft = buffer.length( );
    const char *ptr = buffer.c_str( );
    ssize_t nwritten;

    while ( nleft > 0 ) {
        if ( ( nwritten = Send( ptr, nleft ) ) < 0 ) {
            return -1;
        }
        nleft -= nwritten;
        ptr += nwritten;
    }

    return buffer.length();
}

ssize_t
nestProtocol::Receiven( char *buffer, ssize_t n ) {

    ssize_t nleft = n;
    char *ptr = buffer;
    ssize_t nread;

    while( nleft > 0 ) {
        if ( ( nread = Receive( ptr, nleft ) ) < 0 ) {
            nest_perror( "read" );
            return -1;
        } else if ( nread == 0 ) {
            break;        // EOF
        }
    
        nleft -= nread;
        ptr   += nread;
    }

    return( n - nleft );
}
