/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent,
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * For contact information, please visit:
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_PROTOCOL_FILESYSTEM_H
#define _NEST_PROTOCOL_FILESYSTEM_H

#include "nest_common.h"
#include "nest_protocol_storage.h"
#include "nest_speak.i.h"

#include "nest_pread.h"

class nestProtocolFilesystem : public nestProtocolStorage {
public:


    nestProtocolFilesystem( int filedescriptor );

    nestProtocolFilesystem( int filedescriptor, off_t offset );

    virtual ~nestProtocolFilesystem();

    virtual ssize_t Receive( char *buffer, ssize_t n );

    virtual ssize_t Send( const char *buffer, ssize_t n );

    virtual int getType( ) { return STORAGE_PROTOCOL; }

    virtual bool hasSocket() { return true; }

    int getSocket() { return filedescriptor; }

    virtual nestProtocol *clone();


    virtual int  set_filenames(char *logical_name, char *physical_name);

    virtual int  get_filenames(char *logical_name, char *physical_name);

    //virtual int  set_offset( off_t offset ); 

    virtual int  set_open_flags(int flags);

    virtual int  get_open_flags(int *flags);


    virtual int  set_open_filesize(off_t size);

    virtual int  get_open_filesize(off_t *size);


    virtual int  set_ids(user_id_t uid, user_id_t gid);

    virtual int  get_ids(user_id_t *uid, user_id_t *gid);


    virtual NestReplyStatus receive_state( int pipe );

    virtual NestReplyStatus send_state( int pipe );

    virtual char * get_fdkey();

    virtual int set_fdkey( char *fdkey );

        // call this to tell the connection not to close the fd
    virtual void set_fd_cache_status( bool status );

    virtual bool get_fd_cache_status( );

private:
    void init( int filedescriptor );
    
    int     filedescriptor;
    off_t   offset;

    bool    use_pread;
    
    int     open_flags;
    off_t   open_filesize;
    char    logical_filename[NEST_MAXPATHLEN],
            physical_filename[NEST_MAXPATHLEN];
    char    *fdkey;

    bool    fd_is_cached;
    user_id_t   uid, gid;
};

#endif
