/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_PROTOCOL_GRID_FTP_H
#define _NEST_PROTOCOL_GRID_FTP_H

#include "nest_common.h"
#include "nest_protocol.h"
#include "nest_globus.h"

#include "nest_storage_manager.h"

#ifndef HAVE_PTHREAD
// if we're using the non-threaded globus libs
#include "nest_pthread_util.h"
#endif


class nestProtocolGridFtp : public nestProtocol {
public:
    static int listen( int port );

    static nestProtocol* accept( int socket );
    
    nestProtocolGridFtp( globus_ftp_control_handle_t *handle, int id ); 
    
    virtual ~nestProtocolGridFtp();
    
    virtual bool sendPwd( const char *pwd );
    
    virtual bool sendDirectoryList(  NestFileInfo *list );
    
    virtual bool sendErrorMessage(  const char* filename, const char* func_name, int error );
    
    virtual bool sendFinalAck(  int transferType );

    virtual bool sendTransferInfo(  nestRequest *req );

    virtual bool sendFilesize( long size);

    virtual nestRequest* receiveRequest(  fd_set *ready );

    virtual ssize_t Receive( char *buffer, ssize_t n );

    virtual ssize_t Send( const char *buffer, ssize_t n );
    
    virtual int getType( ) { return GFTP_PROTOCOL; }

    virtual bool hasSocket() { return false; }

    virtual NestReplyStatus gssAccept( char **username, char **globusname );

    int getSocket() { return -1; }
    
    virtual bool endThirdPartyTransfer( int ) { assert( 0 ); return false; }

    globus_ftp_control_handle_t * getHandle();

private:

    globus_ftp_control_handle_t     *handle;
    nestRequest                     *request;
    char                            *username;
    char                            *globusname;
    int                             id;
    int                             reqType;
    char                            buffer[FILEBUFFER];
    int                             bufsize;
    int                             transmissionLength;
    bool                            isNew;
    bool                            eof;
    globus_off_t                    offset;
    gss_cred_id_t                   delegated_credential_handle;
    globus_result_t                 transferStatus;
    globus_bool_t                   authResult;
    globus_bool_t                   acceptResult;

    bool isNewConnection();
    
    static globus_fifo_t fifo;
    static pthread_mutex_t fifo_mutex;
    static int pipes[2];
 
    static void commandCallback( void *arg, 
            struct globus_ftp_control_handle_s *handle,
            globus_object_t *error );

    static void closeCallback( void *arg, 
            struct globus_ftp_control_handle_s *handle,
            globus_object_t *error,
            globus_ftp_control_response_t *response );
    
    static void controlDataCallback( void *callback_arg, 
                    globus_ftp_control_handle_t * handle, 
                    globus_object_t * error, 
                    globus_byte_t * buffer, 
                    globus_size_t length, 
                    globus_off_t offset, 
                    globus_bool_t eof );
    
    static void signalNest();

    static void clearNestSignal();
    
    static void * GlobusCondWait( void *);

    static char * globusResultToString( globus_result_t result );

    static void dataConnectCallback(  void *callback_arg,
                        struct globus_ftp_control_handle_s *handle,
                        unsigned int stripe_ndx,
                        globus_bool_t reused,
                        globus_object_t *error);
    
    static void listenCallback( void *args, globus_ftp_control_server_t *handle,
            globus_object_t *error );

    static void authCallback( void *args, globus_ftp_control_handle_t *handle,
            globus_object_t *error, globus_ftp_control_auth_info_t * result );
    
    static void acceptCallback( void *args, globus_ftp_control_handle_t *handle,
        globus_object_t *error );

    static void readCommands( void *args, globus_ftp_control_handle_t *handle,
        globus_object_t *error, 
        globus_ftp_control_command_t *command ); 

    static int getAuthRequirements( );
    
    void assignRequest( nestRequest *request );

    nestRequest * getRequest( );
    
    static void lockFifo();

    static void unlockFifo();

    static int enQueueConnection( nestProtocol *connection );

    static nestProtocol * deQueueConnection( );
    
    virtual bool sendAck(  );

    virtual bool sendMessage(  const void *vptr );
    globus_result_t closeDataChannel( );

};

#endif
