/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_PROTOCOL_MANAGER_H
#define _NEST_PROTOCOL_MANAGER_H

#include "nest_common.h"
#include "nest_request.h"
#include "nest_speak.h"
#include "nest_speak.i.h"
#include "nest_client.h"

#include "nest_protocol_nest.h"
#include "nest_protocol_http.h"
#include "nest_protocol_gridftp.h"
#include "nest_protocol_nfs.h"
#include "nest_protocol_proxy.h"
#include "nest_protocol_filesystem.h"
#include "nest_protocol_advertise.h"

#define   FTP_SERVER_PORT   21
#define  HTTP_SERVER_PORT   80
#define  GFTP_SERVER_PORT 2811
#define  NEST_SERVER_PORT 7878
#define PROXY_SERVER_PORT    0  // off by default.  Turn on for proxies.
#define   NFS_SERVER_PORT 2049 


class
nestProtocolManager {

public:

    static int init( );

    static nestProtocol* getProtocolByType  ( int protocol_type   );
    static nestProtocol* getProtocolBySocket( int protocol_socket );

    static NestReplyStatus initiateThirdPartyTransfer( nestClient *client, 
            nestProtocol **connection, char *protocol );

    static nestClient* accept(  int protocol_socket,
                                fd_set *readies,
                                int accepts );
    
    static nestProtocol* newConnection( int csd, int protocol );

    static NestReplyStatus SendConnection( int sd, nestProtocol *connection );

    static nestProtocol *ReceiveConnection( int sd, int protocol );
    
    static int getProtocolFromSocket( int protocol_socket ); 
    static int getProtocolPortFromSocket( int protocol_socket );
    static char * getProtocolNameFromSocket( int protocol_socket ); 
    static char * protocolName( int protocol );

    static NestReplyStatus getCredentialInfo( char **subject, long *expires );

    static bool blockingTransfer( nestProtocol *protocol, int type, int fd, 
            long len );

    static int firstProtocolSocket( );
    static int  lastProtocolSocket( );
    static int  nextProtocolSocket( );
    
    static int  NestsetSocket( int domain, int type, int protocol, 
            int my_protocol);

    static void setMaxsocket( ); 
    static int  maxSocket( );

    static void getCredential( );

private:

    static void reportOpenPort( char *protocol, int port, int status );
};


#endif
