/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_PROTOCOL_NFS_H
#define _NEST_PROTOCOL_NFS_H

#include <queue>

#include "nest_common.h"
#include "nest_protocol.h"

#include "nfs_prot.h"

class nestProtocolNFS : public nestProtocol {
public:
    static int listen( int port );

    static nestClient* accept( int socket, 
                                fd_set *readies, 
                                int accepts );
    
    nestProtocolNFS(   SVCXPRT    *transp, 
                        xdrproc_t xdr_result,
                        xdrproc_t xdr_argument,
                        caddr_t   argument,
                        int       general_type,
                        int       request_type );

    virtual ~nestProtocolNFS();

    virtual bool persistent();
    
    virtual bool sendFinalAck( int transferType );
    
    virtual bool sendAck(  );

    virtual bool sendTransferInfo( nestRequest* request );
    
    virtual bool sendErrorMessage(  const char* filename, const char* func_name, int error );
    
    virtual NestReplyStatus sendFileinfo( NestFileInfo *info );

    virtual NestReplyStatus setFileattr( NestFileInfo *info, const char *fname, void *fattr );

    virtual bool sendDiskspace( double avail, double total );
    
    virtual bool sendDirectoryList( NestFileInfo *list );
    
    virtual bool sendFilesize( long size);

    virtual nestRequest* receiveRequest(  fd_set *ready );

    virtual ssize_t Receive( char *buffer, ssize_t n );

    virtual ssize_t Send( const char *buffer, ssize_t n );
    
    virtual int getType( ) { return NFS_PROTOCOL; }

    virtual bool hasSocket();

    NestReplyStatus sendFilehandle( NestFileInfo *handle );

    int getSocket();
    
private:

    static void newconnection(  nestRequest *,
                                SVCXPRT *, 
                                xdrproc_t, 
                                xdrproc_t, 
                                caddr_t,
                                svc_req *, 
                                int );
    
    static void request_handler( svc_req *, SVCXPRT * );
    
    static void nfs_request_handler( svc_req *, SVCXPRT * );

    static void mount_request_handler( svc_req *, SVCXPRT * );
    
    static char *reverseLookup( nfs_fh *fh );
    
    static int pipes[2];

    static queue< nestClient *>clients; 

    NestReplyStatus sendReply( char *result );
    
    NestReplyStatus setFileAttributes( fattr *attributes, NestFileInfo *info );

    void lockMutex();
    void unlockMutex();
    
    SVCXPRT     *transp;
    xdrproc_t   xdr_result;
    xdrproc_t   xdr_argument;
    caddr_t     argument; 
    int         gen_type;
    int         req_type;
};

#endif
