
/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent,
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * For contact information, please visit:
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_STORAGE_FD_CACHE_H
#define NEST_STORAGE_FD_CACHE_H

#include "nest_common.h"

struct ltstr {
    bool operator()(const char* s1, const char* s2) const {
        return strcmp(s1, s2) < 0;
    }
};

#define LIST_ITR list<cache_entry*>::iterator
#define MAP_ITR  multimap<const char*,LIST_ITR,ltstr>::iterator

typedef struct {
    int fd;
    char *key;
} cache_entry;

class
nestFdCache {

public:
    nestFdCache( unsigned max_size );

    ~nestFdCache( );

    bool stow( int fd, const char *key ); 

        // returns fd if found, -1 otherwise
    int  lookup( const char *key ); 

    int getSize( );

    int invalidateAll( const char *subkey );

private:
    void makeSpace();
    void removeEntry( cache_entry *old );

    unsigned max_size;
    list<cache_entry*> cache_list;
    multimap< const char *, LIST_ITR, ltstr > cache_map;
};

#endif
