/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_WORKER_POOL_H
#define _NEST_WORKER_POOL_H

#include "nest_common.h"
#include "nest_worker.h"
#include "nest_protocol_manager.h"
#include "nest_speak.i.h"

enum { WORKER_BUSY, WORKER_IDLE, WORKER_DEAD };

class 
nestWorkerPool {

public:

    nestWorkerPool( int quantity, fd_set * watchfds, 
        int( * ) (int ) );    

    int createWorkers( int quantity, fd_set * watchfds );

    nestWorker* getIdleWorker( );

    int addWorker( pid_t pid, int stream, int id, fd_set *watchfds );

    int removeWorker ( pid_t pid, fd_set *watchfds, fd_set *readyfds );

    nestWorker* getWorker( pid_t pid );

    nestWorker* getWorkerFromID( int id );

    nestWorker* canReceiveMessage( fd_set *readyfds );

    int getMaxfds( );

    ssize_t size( );

    void debugLevel( long level );

    static bool SendWorkerMessage( NestConnection con, int id, int client_id,
            int client_socket, int status, nestFiletransfer *transfer );

    static NestReplyStatus ReceiveWorkerMessage( NestConnection con,
            int &client_id, int &client_socket, struct IO_Measurement * ); 
    

private:

    list< nestWorker* > workers;
    int maxfd;
    int (*worker_loop) (int);    
    int worker_id;
};    



#endif
