/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#include "nest_common.h"
#include "network_limit.h"

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

static void * thread_start( void *arg )
{
    NetworkLimit *n = (NetworkLimit*)arg;
    n->run();
    return 0;
}

NetworkLimit::NetworkLimit( )
{
    pthread_t tid;

    pthread_mutex_init(&mutex,0);
    pthread_cond_init(&cond,0);

    bandwidth = 0;
    interval = 50000;

    nest_pthread_create(&tid,0,thread_start,this);
    pthread_detach(tid);
}

void NetworkLimit::run()
{
    while(1) {
        pthread_mutex_lock(&mutex);
        available = (unsigned long)(((float)bandwidth)*interval/MILLION);
        pthread_cond_broadcast(&cond);
        pthread_mutex_unlock(&mutex);
        NestNanosleep(interval*1000);
    }
}

void NetworkLimit::setBandwidth( unsigned long b )
{
        bandwidth = b;
        nest_printf( "Bandwidth limited to %ld bytes/sec\n", bandwidth );
}

void NetworkLimit::setInterval( unsigned long i )
{
    interval = i;
}

void NetworkLimit::consumeBandwidth( unsigned long bytes )
{
    if ( bandwidth != 0 ) {
        pthread_mutex_lock(&mutex);
        while(bytes>0) {
            if( available>=bytes ) {
                available -= bytes;
                bytes = 0;
                break;
            } else {
                bytes -= available;
                available = 0;
            }
            pthread_cond_wait(&cond,&mutex);
        }
        pthread_mutex_unlock(&mutex);
    }
}
