/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NETWORK_LIMIT_H
#define NETWORK_LIMIT_H

#include <pthread.h>

class NetworkLimit {
public:
    NetworkLimit();

           // Set network bandwidth, measured in bytes per second.
    // Zero means unlimited.
    // Default bandwidth is zero.
    void setBandwidth( unsigned long bytes );

    // Set measurement interval in microseconds.
    // Default interval is 1/4 second.
    void setInterval( unsigned long micros );

    // Declare an intent to consume this many bytes.
    // The caller may be delayed until bandwidth is available.
    void consumeBandwidth( unsigned long bytes );

    // Control producedure run by the contained thread.
    // Don't call this yourself.
    void run();

private:
    unsigned long bandwidth;
    unsigned long available;
    unsigned long interval;

    pthread_mutex_t mutex;
    pthread_cond_t cond;
};

#endif
